import json
from django import forms
from django.template import Template, Context


template_content = Template("""
<br/>
<pre>{{value}}</pre>
<br/>
<table>
    <thead>
        <tr>
            <th>#</th>
            {% for k in keys %}
            <th>{{k}}</th>
            {% endfor %}
            <th></th>
        </tr>
    </thead>
    <tbody>
        {{markup|safe}}
    </tbody>
</table>
<br/>
""")


class LODWidget(forms.Widget):

    def __init__(self, keys, lines):
        """
        Initialize
            :keys
            :lines
            :seperator
        """
        self.keys = keys
        self.lines = lines
        self.separator = "__________"
        super(LODWidget, self).__init__()

    def value_from_datadict(self, data, files, name):
        """ Create JSON text from POST Dictionary """
        sep = self.separator
        keys = self.keys
        value = []
        for i in range(0, self.lines):
            dic = {k: data.get("%d%s%s" % (
                i, sep, k)) for k in keys}
            if all([not x for x in dic.values()]):
                continue
            value.append(dic)
        return json.dumps(value)

    def render(self, name, value, attrs=None):
        """ Render HTML Input for Field """
        sep = self.separator
        keys = self.keys

        value = value or []
        values = []
        for i in range(0, self.lines):
            values.append([])
            for key in keys:
                k = "%d%s%s" % (i, sep, key)
                v = value[i].get(key) or "" if i < len(value) else ""
                values[i].append((k, v))

        markup = "".join(
            """
            <tr>
                <td style="vertical-align:middle">%d</td>
                %s
            </tr>
            """ % (i+1, "".join([
                """
                <td style="vertical-align:middle">
                    <input name="%s" value="%s" />
                </td>
                """ % (k, v)
                for k, v in val]))
            for i, val in enumerate(values)
        )

        data = {
            'keys': keys,
            'value': value,
            'markup': markup
        }
        html = template_content.render(Context(data))
        return html



from django.db import models
from django.core.exceptions import ValidationError
import json
import datetime
from django.utils.translation import ugettext_lazy as _
from widgets import LODWidget


# Serialize datetime which may be present in the JSON
json.JSONEncoder.default = lambda self, obj: (obj.isoformat() if
    isinstance(obj, datetime.datetime) else None)


def validate_object(value, fields):
    """
    Validate the Python object and
    raise errors in case of invalid
    keys in the list of dictionaries
    """
    for i, val in enumerate(value):
        val_keys = val.keys()

        # Check if the keys are Valid
        for k in val_keys:
            if k not in fields:
                raise ValidationError(_("Invalid Key: %s in index: %d" % (str(k), i)))

        # Check if a key is missing
        for f in fields:
            if f not in val_keys:
                raise ValidationError(_("Key: %s is missing in index: %d" % (f, i)))
    return False


class LODField(models.Field):
    """
    A Model field to represent a List of
    Python Dictionaries
    """
    description = "List of Python Dictionaries"

    def __init__(self, *args, **kwargs):
        """
        Initialize
            :fields - A list of Key names
            :lines - Max number of Dictionaries (Djago Admin)
            :max_length - Size of underlying VARCHAR
        """
        self.fields = kwargs.pop('keys', ["Key", "Value"])
        self.lines = kwargs.pop('lines', 10)
        self.char_size = kwargs.pop('max_length', 1000000)

        if type(self.fields) != list:
            raise ValidationError(_("Invalid value: fields"))

        if type(self.char_size) != int:
            raise ValidationError(_("Invalid value: char_size"))

        if type(self.lines) != int:
            raise ValidationError(_("Invalid value: lines"))

        if any([type(k) != str for k in self.fields]):
            raise ValidationError(_("All Keys should be string"))

        super(LODField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        """ Deconstructor function """
        name, path, args, kwargs = super(LODField,
            self).deconstruct()
        return name, path, args, kwargs

    def db_type(self, connection):
        """ Underlying SQL Column type """
        return 'VARCHAR(%s)' % self.char_size

    def from_db_value(self, value, expression, connection, context):
        """ Convert data from DB """
        return self.to_python(value=value)

    def get_prep_value(self, value):
        """ Prepare object to be saved to DB """
        validate_object(value=value, fields=self.fields)
        return json.dumps(value or [])

    def to_python(self, value):
        """ Convert to Python Object """
        return json.loads(value or '[]')

    def pre_save(self, model_instance, add):
        """ Convert to Saved format """
        value = getattr(model_instance, self.attname)
        value = self.to_python(value=json.dumps(value or []))
        setattr(model_instance, self.attname, value)
        return value

    def formfield(self, **kwargs):
        """ Set the custom form widget for this field """
        kwargs['widget'] = LODWidget(keys=self.fields,
            lines=self.lines)
        return super(LODField, self).formfield(**kwargs)



#!/usr/bin/env python
"""
Django-LODField
"""

from setuptools import setup, find_packages
from os import path
# from codecs import open

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.txt')) as f:
    long_description = f.read()

# Get the long description from the README file
# with open(path.join(here, 'README.md'), encoding='utf-8') as f:
#     long_description = f.read()

setup(
    name='django-lodfield',
    version='1.0.3',

    description='List of Dictionaries Model Field for Django',
    long_description=long_description,

    url='http://nakule.in/django-lodfield/',

    author='Nakul Ezhuthupally Sibiraj',
    author_email='nakule@gmail.com',

    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',

        'Framework :: Django',
        'Framework :: Django :: 1.4',
        'Framework :: Django :: 1.5',
        'Framework :: Django :: 1.6',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries :: Python Modules',


        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7'
    ],

    keywords='python developer web django models admin',

    packages=find_packages(exclude=[]),
)
